@@ -7,6 +7,28 @@ class baseMod():
         self.menuInfo = menuInfo
         self.msgr = menuInfo['sm']
         self.dev = self.msgr.shipei.name
+        self.tag = self.msgr.shipei.tag
+
+        ##初始化保存数据的路径
+        self.path = ""
+        import os
+        if self.dev == "smp":
+            if os.path.isdir('../Sandbox/'):
+                self.path = '../Sandbox/'
+            elif os.path.isdir('../../Sandbox/'):
+                self.path = '../../Sandbox'
+        elif self.dev == "win":
+            if os.path.isdir('../DAKu/testdata/'):
+                self.path = '../DAKu/testdata/'
+            elif os.path.isdir('../../DAKu/testdata/'):
+                self.path = '../../DAKu/testdata/'
+        if not self.ctag('board'):
+            import os
+            if not os.path.isdir(self.path):
+                if self.path:
+                    os.mkdir(self.path)
+    def ctag(self,str):
+        return str in self.tag
 
     def start(self):
         self.show()
@@ -19,6 +41,19 @@ class baseMod():
         self.backToMenu()
     def output(self,str):
         self.msgr.output(str)
+    def saveCSV(self,slist,backstr):
+        if self.ctag('board'):
+            return     
+        import pandas as pd
+        df = pd.DataFrame(slist)
+        from datetime import datetime
+        fn = self.path + datetime.now().strftime('%y%m%d-%H%M%S')+backstr
+        df.to_csv(fn,index=False)
+    def saveTXT(self,textstr,backstr):
+        from datetime import datetime
+        sfn = self.path + datetime.now().strftime('%y%m%d-%H%M%S')+backstr
+        with open(sfn,'w') as f:
+            f.write(textstr)        
 
 #基础的循环采集模式
 #只需要重载loopUnit函数即可
@@ -33,7 +68,7 @@ class loopMod(baseMod):
         self.output("Looping...")
     def loop(self,_=None):
         while not self.stop:
-            while self.pause:
+            while self.pause and not self.stop:
                 pass
                 time.sleep(self.pauseWait)
             self.loopUnit()
@@ -42,21 +77,60 @@ class loopMod(baseMod):
         self.backToMenu()
         self.msgr.workModule.show()
     def show(self):
-        if self.dev != 'zkb':
+        if self.ctag('board'):
+            import _thread as th
+            th.start_new_thread(self.loop,(None,))
+        else:
             import threading
             t = threading.Thread(target=self.loop)
             t.start()
-        else:
-            import _thread as th
-            th.start_new_thread(self.loop,(None,))
+
     def InputKey(self,key):
         if not key:
             return
-        if key in "UAPHD":
+        if key in "UAPHDT":
             self.pause = not self.pause
-        elif key in "LBNY":
+        elif key in "LRBNOY":
             self.stop = True
 
+#高级循环
+class loopPro(baseMod):
+    def __init__(self,mi):
+        baseMod.__init__(self,mi)
+        self.pause = False
+        self.stop = False
+        self.isRunning = False
+    def loop(self):
+        pass
+
+    def running(self,_=None):
+        self.isRunning = True
+        self.loop()
+        self.isRunning = False
+
+    def show(self):
+        if self.ctag('board'):
+            import _thread as th
+            th.start_new_thread(self.running,(None,))
+        else:
+            import threading
+            t = threading.Thread(target=self.running)
+            t.start()
+
+    def InputKey(self,key):
+        if not key:
+            return
+        if self.isRunning:
+            if key in "UAPHDT":
+                self.pause = not self.pause
+            elif key in "LRBNOY":
+                self.stop = True
+        else:
+            if key in "UAPHDT":
+                self.show()
+            elif key in "LRBNOY":
+                self.backToMenu()
+
 #关机面板：内容为纯黑屏        
 #暂时退出，关闭屏幕，按下任意键重新启动
 class offStatus(baseMod):
@@ -99,8 +173,10 @@ class menu():
         if self.submenu:
             menuItem = self.data
             for i in self.submenu:
-                mtitle = list(menuItem.keys())[i]
-                menuItem = menuItem[list(menuItem.keys())[i]]
+                keys = list(menuItem.keys())
+                keys.sort()
+                mtitle = keys[i]
+                menuItem = menuItem[keys[i]]
             mtitle += '\n'
         else:
             mtitle = '主菜单\n'
@@ -135,9 +211,9 @@ class menu():
         if not key:
             return
         self.msgr.msg=""
-        if key in "UuH":
+        if key in "UH":
             self.movePointer(-1)
-        elif key in "DdTAP":
+        elif key in "DTAP":
             self.movePointer(1)
         elif key in "ROBN":
             if self.enterSub():
