@@ -16,6 +16,7 @@ class starMessager:
         self.workModule = None
         self.wsStat = "waiting"
         self.wsMsg = ""
+        self.closedWs = []
         self.shipei = wa
     def output(self,str):
         if sm.shipei.name == 'win':
@@ -53,24 +54,48 @@ if sm.shipei.name == 'zkb':
     button_a.event_pressed = inputA
     button_b.event_pressed = inputB
 
+if sm.shipei.name == 'wlb':
+    def wlbKey(_):
+        from future import sensor
+        while True:
+            sm.shipei.Sleep(0.2)
+            if sensor.btnValue("a"):
+                sm.workModule.InputKey("A")
+            elif sensor.btnValue("b"):
+                sm.workModule.InputKey("B")
+
+    import _thread as th
+    th.start_new_thread(wlbKey,(None,))
+
 ########## ws服务器实体  ################
-if sm.shipei.name != 'zkb':
+class webConfig():
+    def __init__(self):
+        self.wsPort = '8525'
+        self.httpPort = '8081'
+        self.webPath = './stardevice'
+        self.bdzPath = '/sdcard/qpython/projects3/WalArtMinS/stardevice/'
+webConf = webConfig()
+
+if not sm.shipei.name in ['zkb','wlb']:
     import asyncio
     import websockets
     import json
 
-    wsPort = '8525'
-
     def d2j(dict):
         return json.dumps(dict,ensure_ascii=False)
 
     ScreenSize = (240,120)
     async def sendscreen(websocket,svgContent):
         try:
-            from gTixi import SvgScreen
+            from . import SvgScreen
         except:
             from wadapter import SvgScreen
-        await websocket.send(d2j({"type":"screen","msg":SvgScreen(ScreenSize).DrawStdLines(svgContent)}))
+        try:
+            await websocket.send(d2j({"type":"screen","msg":SvgScreen(ScreenSize).DrawStdLines(svgContent)}))
+            return True
+        except:
+            print("Connection Closed...")
+            return False
 
     if sm.shipei.name == 'smp':
         def hardIn():
@@ -82,27 +107,27 @@ if sm.shipei.name != 'zkb':
         hin = threading.Thread(target=hardIn)
         hin.start()
 
-    async def sendKey():
+    async def sendKey(websocket):
         while True:
-            while sm.stat == "waiting":
-                import time
+            while sm.stat == "waiting" and id(websocket) not in sm.closedWs:
                 await asyncio.sleep(0.05)
             if sm.stat == "key":
                 sm.workModule.InputKey(sm.msg)
             sm.stat = "waiting"
-            if sm.wsStat == "break":
-                sm.wsStat="waiting"
+            if id(websocket) in sm.closedWs:
                 break
 
     async def webOut(websocket):
         while True:
-            while sm.wsStat == "waiting":
+            while sm.wsStat == "waiting" and id(websocket) not in sm.closedWs:
                 await asyncio.sleep(0.05)
-            if sm.wsStat == "screen":        
-                await sendscreen(websocket,sm.wsMsg)
+            if sm.wsStat == "screen":
+                sm.wsStat = "waiting"
+                ss = await sendscreen(websocket,sm.wsMsg)
+                if not ss:
+                    sm.closedWs.append(id(websocket))
             sm.wsStat = "waiting"
-            if sm.wsStat == "break":
-                sm.wsStat="waiting"
+            if id(websocket) in sm.closedWs:
                 break
 
     async def webIn(websocket):
@@ -110,7 +135,7 @@ if sm.shipei.name != 'zkb':
             try:
                 message = await websocket.recv()
             except:
-                sm.wsStat="break"
+                sm.closedWs.append(id(websocket))
                 break
             message = json.loads(message)
             if message['type'] == 'key':
@@ -121,13 +146,17 @@ if sm.shipei.name != 'zkb':
         await sendscreen(websocket,"欢迎来到星尘演示机\n")
         await asyncio.sleep(1)
         sm.workModule.show()
-        await asyncio.gather(webIn(websocket),webOut(websocket),sendKey())
+        await asyncio.gather(webIn(websocket),webOut(websocket),sendKey(websocket))
+        sm.closedWs.remove(id(websocket))
         
     loop = asyncio.new_event_loop()
     print('Star Socket Server Start!')
     print('---------------------------')
     asyncio.set_event_loop(loop)
-    loop.run_until_complete(websockets.serve(echo, '0.0.0.0', wsPort))
+    try:
+        loop.run_until_complete(websockets.serve(echo, '0.0.0.0', webConf.wsPort))
+    except:
+        print("Port {} is binded to wsServer!".format(webConf.wsPort))
     def wsStart():
         loop.run_forever()
     def wsStop():
@@ -157,7 +186,7 @@ else:
 """
 This is a sample for qpython webapp
 """
-if sm.shipei.name != 'zkb':#只有bdz需要自己建立bottle架构的网络服务器
+if not sm.shipei.name in ['zkb','wlb']:#只有bdz需要自己建立bottle架构的网络服务器
     from bottle import Bottle, ServerAdapter
     from bottle import run, debug, route, error, static_file, template, post
 
@@ -182,7 +211,7 @@ if sm.shipei.name != 'zkb':#只有bdz需要自己建立bottle架构的网络服
             #self.server.shutdown()
             self.server.server_close() #<--- alternative but causes bad fd exception
             print("# qpyhttpd stop")
-
+    
     def httpSrv():
         ######### BUILT-IN ROUTERS ###############
         @route('/__exit', method=['GET','HEAD'])
@@ -193,18 +222,18 @@ if sm.shipei.name != 'zkb':#只有bdz需要自己建立bottle架构的网络服
         @route('/<filepath:path>')
         def serverstatic(filepath):
             if sm.shipei.name == 'bdz':
-                return static_file(filepath, root='/sdcard/qpython/projects3/WalArtMinS/stardevice/')
+                return static_file(filepath, root=webConf.bdzPath)
             else:
-                return static_file(filepath,root='./stardevice')
+                return static_file(filepath,root=webConf.webPath)
 
 
         ######### WEBAPP ROUTERS ###############
         @route('/')
         def home():
             if sm.shipei.name == 'bdz':
-                return static_file("index.html", root='/sdcard/qpython/projects3/WalArtMinS/stardevice/')
+                return static_file("index.html", root=webConf.bdzPath)
             else:
-                return static_file("index.html",root='./stardevice')
+                return static_file("index.html",root=webConf.webPath)
         @post('/deviceStart')
         def startws():
             wsCtrl().start()
@@ -218,7 +247,7 @@ if sm.shipei.name != 'zkb':#只有bdz需要自己建立bottle架构的网络服
         app.route('/<filepath:path>', method='GET')(serverstatic)
         app.post('/deviceStop',method='POST')(stopws)
         app.post('/deviceStart',method='POST')(startws)
-        server = MyWSGIRefServer(host="0.0.0.0", port="8081")
+        server = MyWSGIRefServer(host="0.0.0.0", port=webConf.httpPort)
         app.run(server=server,reloader=False)
     import threading
     httpServer = threading.Thread(target=httpSrv)
