@@ -76,11 +76,6 @@ Revisions
 220331 numpy, smbus for qpython
 
 """
-try:
-    from . import alib, waFile, waText
-    from .alib import alib
-except:
-    from alib import alib,waFile,waText
 aWaEnv={'sWal':'WalArtmin'}
 
 a支持的平台='''[Python3|3.6+在电脑和GT0上运行的]
@@ -199,7 +194,11 @@ class Shipei(): #统一Fascade
         self.aenv=aenv # aWaEnv
         self.i2c = None
         self.name = "default"
-
+        self.tag = []
+    def ctag(self,str):
+        return str in self.tag
+    def gpio(self,pin,value= None):
+        return None
     def DispChar(self,txt):
         if isinstance(txt,str):
             lines=txt.split('\n')
@@ -278,6 +277,9 @@ class Shipei(): #统一Fascade
         print('Done scan I2C')
         return lsI2c
 class SpEsp32(Shipei): #Esp32, includes Zkb, Wlb etc.
+    def __init__(self,aenv):
+        Shipei.__init__(self,aenv)
+        self.tag.extend(['board'])
     def ASetupI2c(self):
         # 使用self.oi2c
         def wbd(address,pos,dat,i2c=self.oi2c):
@@ -291,15 +293,17 @@ class SpEsp32(Shipei): #Esp32, includes Zkb, Wlb etc.
         def ribd(address,pos,iLen,i2c=self.oi2c):
             r=i2c.readfrom_mem(address,pos,iLen)
             return r
+        def wtm(address,pos,dat,i2c=self.oi2c):
+            return i2c.writeto_mem(address,pos,(bytearray(dat)))
         class AI2C():
-            def __init__(self,write_byte_data,read_byte_data,read_word_data,read_i2c_block_data):
+            def __init__(self,write_byte_data,read_byte_data,read_word_data,read_i2c_block_data,write_to_mem):
                 self.read_word_data = read_word_data
                 self.write_byte_data = write_byte_data
                 self.read_byte_data = read_byte_data
                 self.read_i2c_block_data = read_i2c_block_data
-        return AI2C(write_byte_data=wbd, read_byte_data=rbd, read_word_data=rwd,read_i2c_block_data=ribd)
-        return alib(dict(write_byte_data=wbd, read_byte_data=rbd,read_word_data=rwd,
-                            read_i2c_block_data=ribd))
+                self.write_to_mem = write_to_mem
+        return AI2C(write_byte_data=wbd, read_byte_data=rbd, read_word_data=rwd,read_i2c_block_data=ribd,write_to_mem=wtm)
+
     def CheckKey(self):
         return self.cKey
     def LsScanI2c(self):
@@ -320,7 +324,7 @@ class SpEsp32(Shipei): #Esp32, includes Zkb, Wlb etc.
 
 class SpZkb(SpEsp32): #掌控板适配
     def __init__(self,aenv):
-        #SpEsp32.__init__(self,aenv)
+        SpEsp32.__init__(self,aenv)
         self.name="zkb"
         self.aenv=aenv
         from mpython import oled
@@ -334,7 +338,13 @@ class SpZkb(SpEsp32): #掌控板适配
         from mpython import sleep_ms,button_a,button_b,touchPad_P,touchPad_Y,touchPad_T,touchPad_H,touchPad_O,touchPad_N
         while True:
             sleep_ms(100)           
-
+    def gpio(self,pin,value=None):
+        from machine import Pin
+        pinmap={0:Pin.P0,1:Pin.P1,2:Pin.P2,3:Pin.P3}
+        if value:
+            return Pin(pinmap[pin],Pin.OUT).value(value)
+        else:
+            return Pin(pinmap[pin],Pin.IN).value()        
     def DrawText(self,tPos,sTxt,c=None):
         from mpython import oled
         oled.DispChar(sTxt,tPos[0],tPos[1])
@@ -359,6 +369,7 @@ class SpSmp(Shipei): #树莓派适配
     def __init__(self,aenv):
         Shipei.__init__(self,aenv)
         self.name="smp"
+        self.tag.extend(['web','smbus','gpiorpi','pinmap'])
         self.aenv=aenv
         self.cSoftKey=''
         import math
@@ -489,6 +500,7 @@ class SpBdz(Shipei): #GT1 adapter
     def __init__(self,aenv):
         Shipei.__init__(self,aenv)
         self.name="bdz"
+        self.tag.extend(['web','gpio4','smbus'])
         self.aenv=aenv
         self.cSoftKey=''
         try:
@@ -496,10 +508,11 @@ class SpBdz(Shipei): #GT1 adapter
             self.oi2c=SMBus(6)
             i2c=self.oi2c
             class AI2C():
-                def __init__(self,write_byte_data,read_byte_data):
+                def __init__(self,write_byte_data,read_byte_data,read_i2c_block_data):
                     self.write_byte_data = write_byte_data
                     self.read_byte_data = read_byte_data
-            self.i2c=AI2C(write_byte_data=i2c.write_byte_data, read_byte_data=i2c.read_byte_data)
+                    self.read_i2c_block_data = read_i2c_block_data
+            self.i2c=AI2C(write_byte_data=i2c.write_byte_data, read_byte_data=i2c.read_byte_data, read_i2c_block_data=i2c.read_i2c_block_data)
         except:
             print('SMbus2 not available, install it to use it')
             self.oi2c=None
@@ -509,11 +522,7 @@ class SpBdz(Shipei): #GT1 adapter
         while True:
             import time
             time.sleep(100) #by Spectre Lee, 预留给bdz实体键可以使用时
-        ans = None
-        if self.cSoftKey:
-            ans=self.cSoftKey
-            self.cSoftKey=''
-        return ans
+
     def DrawText(self,tPos,sTxt,c=None):
         self.oSvg.DrawText(tPos,sTxt,c)
     def DispClear(self):
@@ -537,56 +546,26 @@ class SpWlb(SpEsp32):
         from future import i2c
         self.oi2c=i2c
         self.i2c=self.ASetupI2c()
-
-        import _thread
-        def ThreadBtnScan(_,wa=self):
-            #loop scan AB虚拟六个键的功能：LRUDOX
-            tButA=False
-            tButB=False #last button state
-            wa.iLastA=100
-            wa.iLastB=100 #rounds since last button up
-            from future import sensor
-            while True:
-                try:
-                    if sensor.btnValue('a'): #button press down
-                        tButA=True
-                        if wa.cKey=='':
-                            if wa.iLastA<3:
-                                wa.cKey='X'
-                            elif wa.iLastB<3:
-                                wa.cKey='L'
-                            else:
-                                wa.cKey='U'
-                    else:
-                        if tButA:
-                            #if released, record last up
-                            wa.iLastA=0
-                        tButA=False
-                    if sensor.btnValue('b'):
-                        tButB=True
-                        if wa.cKey=='':
-                            if wa.iLastB<3:
-                                wa.cKey='O'
-                            elif wa.iLastA<3:
-                                wa.cKey='R'
-                            else:
-                                wa.cKey='D'
-                    else:
-                        if tButB:
-                            wa.iLastB=0
-                        tButB=False
-                    wa.iLastA = wa.iLastA+1 if wa.iLastA<100 else 100
-                    wa.iLastB = wa.iLastB+1 if wa.iLastB<100 else 100
-                    if (not tButA) and (not tButB):
-                        self.cKey=''
-                    wa.Sleep(.05)
-                except OSError as e:
-                    print(e)
-        _thread.start_new_thread(ThreadBtnScan,(1,))
-
         from future import sleep
         self.Sleep=sleep
-
+    def gpio(self,pin,value):
+        from future import MeowPin
+        pinmap={0:'P0',1:'P1',2:'P2',3:'P3'}
+        if value:
+            MeowPin(pinmap[pin],'OUT').setDigital(value)
+        else:
+            MeowPin(pinmap[pin],'IN').getDigital()
+    def str2unicode(self,a):
+        b = ''.join(map(lambda x:('' if len(hex(x))>=4 else '/x0')+hex(x)[2:],a.encode()))
+        i=0
+        splitLen=2
+        chars=[]
+        while b[i*splitLen:]:
+            c = b[i*splitLen:i*splitLen+splitLen]
+            c = int(c,16)
+            chars.append(c)
+            i += 1
+        return chars
     def DrawText(self,tPos,sTxt,c=(255,255,255)):
         from future import screen
         ipos=0
@@ -616,12 +595,23 @@ class SpWlb(SpEsp32):
         screen.refresh()
     def DrawStdLines(self,input):
         return self.oSvg.DrawStdLines(input)
+    def DrawStdLines(self,input):
+        from future import screen
+        screen.sync = 0
+        screen.clear()
+        if input:
+            lines = input.split("\n")
+            for i in range(len(lines)):
+                if i<4:
+                    screen.textCh(self.str2unicode(lines[i]),0,25+20*i,1,(238,238,0) if i==0 else (0,238,238))
+            screen.refresh()
 
 class SpWin(Shipei):
     def __init__(self,aenv):
         Shipei.__init__(self,aenv)
         self.name = "win"
         self.oSvg = SvgScreen()
+        self.tag.extend(['web','debug'])
     def CheckKey(self):
         while True:
             import time
@@ -637,15 +627,14 @@ class SpWin(Shipei):
 
 def SIp():
     #get Ip address
-    import socket
     try:
+        import socket
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         s.connect(('8.8.8.8', 80))
         ip = s.getsockname()[0]
-    except:
-        ip='#no network connection'
-    finally:
         s.close()
+    except:
+        ip='no network'
     aWaEnv['sIp']=ip
     return ip
 def Wa(aenv=aWaEnv):
@@ -656,14 +645,14 @@ def Wa(aenv=aWaEnv):
     elif 'Future' in aenv['sMachine']:
         return SpWlb(aenv)
     elif 'v7l' in aenv['sMachine']:
-        if 'localhost' in aenv['sMachine']:
+        if 'localhost' in aenv['sNodename']:
             return SpBdz(aenv)
         else:
             return SpSmp(aenv)
     else:
         #print(aenv.sMachine)
         return SpWin(aenv)
-
+SIp()
 Hello()
 wa=Wa()
 print(wa.name)
